package pwc.taxtech.atms.vat.service.impl;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.CellDataSourceType;
import pwc.taxtech.atms.constant.enums.EnumOperationType;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.constant.enums.KeyValueConfigResultType;
import pwc.taxtech.atms.dpo.ReportDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.vatdto.CellCalcInfoDto;
import pwc.taxtech.atms.dto.vatdto.CellDataDto;
import pwc.taxtech.atms.dto.vatdto.CellTemplateReferenceDto;
import pwc.taxtech.atms.dto.vatdto.DataSourceDto;
import pwc.taxtech.atms.dto.vatdto.DataSourceDtoExtend;
import pwc.taxtech.atms.dto.vatdto.ManualDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.ReportDataDto;
import pwc.taxtech.atms.dto.vatdto.VatEnterpriseAccountResultDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.NotSupportedException;
import pwc.taxtech.atms.service.impl.CellConfigTranslater;
import pwc.taxtech.atms.vat.dao.PeriodTaxPayerReportRuleMapper;
import pwc.taxtech.atms.vat.dao.PeriodTaxRuleSettingMapper;
import pwc.taxtech.atms.vat.dpo.DataSourceCellDataDto;
import pwc.taxtech.atms.vat.dpo.DataSourceExtendDto;
import pwc.taxtech.atms.vat.dpo.InputVATInvoiceItemExtendDto;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.ReportService;
import pwc.taxtech.atms.vat.service.impl.report.functions.FormulaHelper;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Component
public class ReportServiceImpl extends VatAbstractService implements ReportService {
    @Autowired
    private PeriodTaxRuleSettingMapper periodTaxRuleSettingMapper;
    @Autowired
    private PeriodTaxPayerReportRuleMapper periodTaxPayerReportRuleMapper;

    @Override
    public OperationResultDto<List<ReportDto>> getReportTemplate(String projectId, EnumServiceType serviceType, Integer periodParam) {
        int period = periodParam != null ? periodParam : 0;
        OperationResultDto<List<ReportDto>> operationResult = new OperationResultDto<>();
        try {
            Project project = projectMapper.selectByPrimaryKey(projectId);
            String organizationId = project == null ? null : project.getOrganizationId();
            if (organizationId == null) {
                operationResult.setResult(false);
                operationResult.setResultMsg("NoOrganization");
                return operationResult;
            }

            Long templateGroupId;

        /*
        对于VAT，从Rule Engine中获取TemplateGroupId
        对于其他Service Type，直接从ProjectServiceType中获取
         */
            if (serviceType.equals(EnumServiceType.VAT)) {
                TaxPayerReportRuleExample example = new TaxPayerReportRuleExample();
                example.createCriteria().andOrganizationIdLike(organizationId);
                List<TaxPayerReportRule> taxPayerReportRules = taxPayerReportRuleMapper.selectByExample(example);
                Optional<TaxPayerReportRule> taxPayerReportRule = taxPayerReportRules.stream().findFirst();
                if (!taxPayerReportRule.isPresent()) {
                    Optional<TaxPayerReportRule> defaultRule = taxPayerReportRules.stream().filter(x -> x.getIsDefault()).findFirst();
                    if (!defaultRule.isPresent()) {
                        operationResult.setResultMsg("NoDefaultRule");
                        return operationResult;
                    }

                    templateGroupId = defaultRule.get().getTemplateGroupId();
                } else {
                    templateGroupId = taxPayerReportRule.get().getTemplateGroupId();
                }

            } else {
                String serviceTypeStr = serviceType.getCode().toString();
                ProjectServiceTypeExample projectServiceTypeExample = new ProjectServiceTypeExample();
                projectServiceTypeExample.createCriteria().andServiceTypeIdEqualTo(serviceTypeStr).andProjectIdEqualTo(projectId);
                Optional<Long> tempVlaue = projectServiceTypeMapper.selectByExample(projectServiceTypeExample).stream().map(ProjectServiceType::getTemplateGroupId).findFirst();
                if (tempVlaue.isPresent()) {
                    templateGroupId = tempVlaue.get();
                } else {
                    templateGroupId = 0L;
                }
            }

            Map<String, Object> map = new HashMap<>();
            map.put("period", period);
            map.put("projectId", projectId);
            map.put("templateGroupID", templateGroupId);

            List<ReportDto> reportDtos = templateMapper.getTemplateLeftJoinReport(map);
            if (reportDtos.isEmpty()) {
                operationResult.setResultMsg("No Template");
                return operationResult;
            }

            operationResult.setResult(true);
            operationResult.setData(reportDtos);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResult.setResult(false);
        }
        return operationResult;
    }

    private void clearPeriodData(String projectId, Integer period, List<Long> templateIds) {
        if (templateIds.isEmpty()) templateIds.add(Long.MAX_VALUE);
        PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
        periodFormulaBlockExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodFormulaBlockMapper.deleteByExample(periodFormulaBlockExample);

        PeriodTaxRuleSettingExample periodTaxRuleSettingExample = new PeriodTaxRuleSettingExample();
        periodTaxRuleSettingExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodTaxRuleSettingMapper.deleteByExample(periodTaxRuleSettingExample);

        PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
        periodCellTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andReportTemplateIdNotIn(templateIds);
        periodCellTemplateMapper.deleteByExample(periodCellTemplateExample);

        PeriodCellTemplateConfigExample periodCellTemplateConfigExample = new PeriodCellTemplateConfigExample();
        periodCellTemplateConfigExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andReportTemplateIdNotIn(templateIds);
        periodCellTemplateConfigMapper.deleteByExample(periodCellTemplateConfigExample);

        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andTemplateIdNotIn(templateIds);
        periodTemplateMapper.deleteByExample(periodTemplateExample);

        PeriodTaxPayerReportRuleExample periodTaxPayerReportRuleExample = new PeriodTaxPayerReportRuleExample();
        periodTaxPayerReportRuleExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        periodTaxPayerReportRuleMapper.deleteByExample(periodTaxPayerReportRuleExample);

        PeriodDataSourceExample periodDataSourceExample = new PeriodDataSourceExample();
        periodDataSourceExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);
        List<PeriodDataSource> periodDataSources = periodDataSourceMapper.selectByExample(periodDataSourceExample);
        List<Long> periodDsIds = Lists.newArrayList(Long.MAX_VALUE);
        periodDsIds.addAll(periodDataSources.stream().map(m -> m.getId()).collect(Collectors.toList()));
        PeriodDataSourceDetailExample periodDataSourceDetailExample = new PeriodDataSourceDetailExample();
        periodDataSourceDetailExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period)
                .andDataSourceIdIn(periodDsIds);

        periodDataSourceDetailMapper.deleteByExample(periodDataSourceDetailExample);
        PeriodCellDataSourceExample periodCellDataSourceExample = new PeriodCellDataSourceExample();
        periodCellDataSourceExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period)
                .andDataSourceIdIn(periodDsIds);
        periodCellDataSourceMapper.deleteByExample(periodCellDataSourceExample);

        periodDataSourceMapper.deleteByExample(periodDataSourceExample);

        PeriodReportExample periodReportExample = new PeriodReportExample();
        periodReportExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period);

        List<PeriodReport> reports = periodReportMapper.selectByExample(periodReportExample);
        List<Long> reportIds = Lists.newArrayList(Long.MAX_VALUE);
        reportIds.addAll(reports.stream().map(m -> m.getId()).collect(Collectors.toList()));
        PeriodCellDataExample periodCellDataExample = new PeriodCellDataExample();
        periodCellDataExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andReportIdIn(
                reportIds);
        periodCellDataMapper.deleteByExample(periodCellDataExample);
        periodReportMapper.deleteByExample(periodReportExample);

    }

    @Override
    public OperationResultDto updateConfig(String projectId, Integer period, Boolean ifDeleteManualDataSource, String generator) {
        OperationResultDto result = new OperationResultDto();
        try {
            if (period == null) {
                result.setResultMsg("peirod is null");
                return result;
            }

            Project project = projectMapper.selectByPrimaryKey(projectId);

            if (project == null) {
                result.setResultMsg("NoProject");
                return result;
            }

            List<Long> exceptTemplateIds = templateMapper.getIdsForExceptTemplate();
//            String strExceptTemplateIds = StringUtils.EMPTY;
//            if (exceptTemplateIds != null && exceptTemplateIds.size() > 0) {
//                strExceptTemplateIds = StringUtils.join(exceptTemplateIds, ",");
//            } else {
//                strExceptTemplateIds = "''";
//            }

            //todo: according to projectId get the templateGroupId
            Long templateGroupId = projectMapper.getTemplateGroupIdByProject(projectId, EnumServiceType.VAT.getCode());
            if (templateGroupId != null && templateGroupId != 0) {
                // 先进行数据清理，包括period开头的所有报表配置表 条件Period
                clearPeriodData(projectId, period, exceptTemplateIds);
                // 根据templategroupid 把 template 插入到 periodTemplate
                TemplateExample example = new TemplateExample();
                example.createCriteria().andTemplateGroupIdEqualTo(templateGroupId);
                List<Template> templates = templateMapper.selectByExample(example);
                List<PeriodTemplate> periodTemplateList = new ArrayList<>();
                int i = 0;
                Long startTimeOut = System.currentTimeMillis();
                logger.debug("templates copy start: " + startTimeOut);
                for (Template template : templates) {
                    i++;
                    Long startTime = System.currentTimeMillis();
                    logger.debug("template copy start: " + startTime);
                    PeriodTemplate periodTemplate = new PeriodTemplate();
                    CommonUtils.copyProperties(template, periodTemplate);
                    periodTemplate.setId(distributedIdService.nextId());
                    periodTemplate.setPeriod(period);
                    periodTemplate.setName(template.getName());
                    periodTemplate.setCode(template.getCode());
                    periodTemplate.setPath(template.getPath());
                    periodTemplate.setReportType(template.getReportType());
                    periodTemplate.setTemplateGroupId(template.getTemplateGroupId());
                    periodTemplate.setOrderIndex(template.getOrderIndex());
                    periodTemplate.setCreateTime(template.getCreateTime());
                    periodTemplate.setUpdateTime(template.getUpdateTime());
                    periodTemplate.setIsSystemType(template.getIsSystemType());
                    periodTemplate.setIsActiveAssociation(template.getIsActiveAssociation());
                    periodTemplate.setParentId(template.getParentId());
                    periodTemplate.setTemplateId(template.getId());
                    periodTemplate.setCreateBy(template.getCreateBy());
                    periodTemplate.setUpdateBy(template.getUpdateBy());
                    periodTemplate.setProjectId(projectId);
                    periodTemplateList.add(periodTemplate);
                    logger.debug("template copy end,used time: " + (System.currentTimeMillis() - startTime) + " ms");
                }
                logger.debug("templates copy end,used: " + (System.currentTimeMillis() - startTimeOut) + " ms, copy: " + i + " items");

                List<Long> templateIdList = periodTemplateList.stream().map(PeriodTemplate::getTemplateId).collect(Collectors.toList());
                CellTemplateExample cellTemplateExample = new CellTemplateExample();
                cellTemplateExample.createCriteria().andReportTemplateIdIn(templateIdList);
                List<CellTemplate> cellTemplateList = cellTemplateMapper.selectByExample(cellTemplateExample);
                List<PeriodCellTemplate> periodCellTemplateList = new ArrayList<>();
                i = 0;
                startTimeOut = System.currentTimeMillis();
                logger.debug("cellTemplates copy start: " + startTimeOut);
                for (CellTemplate cellTemplate : cellTemplateList) {
                    i++;
                    Long startTime = System.currentTimeMillis();
                    //logger.debug("celltemplate copy start: " + startTime);
                    PeriodCellTemplate periodCellTemplate = new PeriodCellTemplate();
                    CommonUtils.copyProperties(cellTemplate, periodCellTemplate);
                    periodCellTemplate.setId(distributedIdService.nextId());
                    periodCellTemplate.setPeriod(period);
                    periodCellTemplate.setReportTemplateId(cellTemplate.getReportTemplateId());
                    periodCellTemplate.setRowIndex(cellTemplate.getRowIndex());
                    periodCellTemplate.setRowName(cellTemplate.getRowName());
                    periodCellTemplate.setColumnIndex(cellTemplate.getColumnIndex());
                    periodCellTemplate.setColumnName(cellTemplate.getColumnName());
                    periodCellTemplate.setComment(cellTemplate.getComment());
                    periodCellTemplate.setCreateTime(cellTemplate.getCreateTime());
                    periodCellTemplate.setUpdateTime(cellTemplate.getUpdateTime());
                    periodCellTemplate.setCellTemplateId(cellTemplate.getId());
                    periodCellTemplate.setDataType(cellTemplate.getDataType());
                    periodCellTemplate.setIsReadOnly(cellTemplate.getIsReadOnly() ? 1 : 0);
                    periodCellTemplate.setCopyFromId(cellTemplate.getCopyFromId());
                    periodCellTemplate.setCreateBy(cellTemplate.getCreateBy());
                    periodCellTemplate.setUpdateBy(cellTemplate.getUpdateBy());
                    periodCellTemplate.setProjectId(projectId);
                    periodCellTemplateList.add(periodCellTemplate);
                    //logger.debug("celltemplate copy end,used time: " + (System.currentTimeMillis() - startTime) + " ms");
                }
                logger.debug("templates copy end,used: " + (System.currentTimeMillis() - startTimeOut) + " ms, copy: " + i + " items");

                List<Long> cellTemplateIdList = periodCellTemplateList.stream().map(PeriodCellTemplate::getCellTemplateId).collect(Collectors.toList());
                CellTemplateConfigExample cellTemplateConfigExample = new CellTemplateConfigExample();
                cellTemplateConfigExample.createCriteria().andReportTemplateIdIn(templateIdList).andCellTemplateIdIn(cellTemplateIdList);
                List<CellTemplateConfig> cellTemplateConfigList = cellTemplateConfigMapper.selectByExample(cellTemplateConfigExample);
                List<PeriodCellTemplateConfig> periodCellTemplateConfigList = new ArrayList<>();
                i = 0;
                startTimeOut = System.currentTimeMillis();
                logger.debug("cellTemplateConfigs copy start:" + startTimeOut);
                for (CellTemplateConfig cellTemplateConfig : cellTemplateConfigList) {
                    i++;
                    Long startTime = System.currentTimeMillis();
                    //logger.debug("cellTemplateConfig copy start: " + startTime);
                    PeriodCellTemplateConfig periodCellTemplateConfig = new PeriodCellTemplateConfig();
                    CommonUtils.copyProperties(cellTemplateConfig, periodCellTemplateConfig);
                    periodCellTemplateConfig.setId(distributedIdService.nextId());
                    periodCellTemplateConfig.setPeriod(period);
                    periodCellTemplateConfig.setCellTemplateId(cellTemplateConfig.getCellTemplateId());
                    periodCellTemplateConfig.setReportTemplateId(cellTemplateConfig.getReportTemplateId());
                    periodCellTemplateConfig.setDataSourceType(cellTemplateConfig.getDataSourceType());
                    periodCellTemplateConfig.setFormula(cellTemplateConfig.getFormula());
                    periodCellTemplateConfig.setParsedFormula(null);
                    periodCellTemplateConfig.setFormulaDescription(cellTemplateConfig.getFormulaDescription());
                    periodCellTemplateConfig.setAccountCodes(cellTemplateConfig.getAccountCodes());
                    periodCellTemplateConfig.setInvoiceType(cellTemplateConfig.getInvoiceType());
                    periodCellTemplateConfig.setTaxRate(cellTemplateConfig.getTaxRate());
                    periodCellTemplateConfig.setInvoiceAmountType(cellTemplateConfig.getInvoiceAmountType());
                    periodCellTemplateConfig.setModelIds(cellTemplateConfig.getModelIds());
                    periodCellTemplateConfig.setCreateBy(cellTemplateConfig.getCreateBy());
                    periodCellTemplateConfig.setCreateTime(cellTemplateConfig.getCreateTime());
                    periodCellTemplateConfig.setUpdateBy(cellTemplateConfig.getUpdateBy());
                    periodCellTemplateConfig.setUpdateTime(cellTemplateConfig.getUpdateTime());
                    periodCellTemplateConfig.setInvoiceCategory(cellTemplateConfig.getInvoiceCategory());
                    periodCellTemplateConfig.setFormulaDataSource(cellTemplateConfig.getFormulaDataSource());
                    periodCellTemplateConfig.setValidation(cellTemplateConfig.getValidation());
                    periodCellTemplateConfig.setParsedValidation(null);
                    periodCellTemplateConfig.setValidationDescription(cellTemplateConfig.getValidationDescription());
                    periodCellTemplateConfig.setVoucherKeyword(cellTemplateConfig.getVoucherKeyword());
                    periodCellTemplateConfig.setCellTemplateConfigId(cellTemplateConfig.getId());
                    periodCellTemplateConfig.setKeyValueParsedFormula(null);
                    periodCellTemplateConfig.setProjectId(projectId);
                    if (cellTemplateConfig.getFormula() != null && cellTemplateConfig.getFormula().contains("@")) {
                        String regex = "@[0-9A-Z.]+";
                        Pattern pp = Pattern.compile(regex);
                        Matcher mm = pp.matcher(cellTemplateConfig.getFormula());
                        String replace_result = cellTemplateConfig.getFormula();
                        Boolean replace = true;
                        while (mm.find()) {
                            KeyValueConfigExample keyValueConfigExample = new KeyValueConfigExample();
                            keyValueConfigExample.createCriteria().andKeyCodeEqualTo(StringUtils.removeStart(mm.group(), "@"));
                            Optional<KeyValueConfig> keyValueConfig = keyValueConfigMapper.selectByExample(keyValueConfigExample).stream().findFirst();
                            if (keyValueConfig.isPresent() && StringUtils.isNotBlank(keyValueConfig.get().getFormula())) {
                                replace_result = replace_result.replace(mm.group(), keyValueConfig.get().getFormula());
                            } else {
                                replace = false;
                            }
                        }

                        if (replace) {
                            periodCellTemplateConfig.setKeyValueParsedFormula(replace_result);
                        }
                        //todo: get the keyvalue from table add to keyvalue parsed formula
//                        KeyValueConfigExample keyValueConfigExample = new KeyValueConfigExample();
//                        keyValueConfigExample.createCriteria().andKeyCodeEqualTo(StringUtils.removeStart(cellTemplateConfig.getFormula(), "@"));
//                        Optional<KeyValueConfig> keyValueConfig = keyValueConfigMapper.selectByExample(keyValueConfigExample).stream().findFirst();
//                        if (keyValueConfig.isPresent()) {
//                            periodCellTemplateConfig.setKeyValueParsedFormula(keyValueConfig.get().getFormula());
//                        }
                    }
                    periodCellTemplateConfigList.add(periodCellTemplateConfig);
                    //logger.debug("cellTemplateConfig copy end,used time: " + (System.currentTimeMillis() - startTime) + " ms");
                }
                logger.debug("cellTemplateConfigs copy end,used: " + (System.currentTimeMillis() - startTimeOut) + " ms, copy: " + i + " items");

                if (periodCellTemplateConfigList != null) {
                    periodCellTemplateConfigList.forEach(a -> {
                        if (StringUtils.isNotBlank(a.getAccountCodes())) {
                            String[] acctCodes = a.getAccountCodes().split(",");
                            EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
                            enterpriseAccountExample.createCriteria().andStdCodeIn(Arrays.asList(acctCodes));
                            List<EnterpriseAccount> enterpriseAccounts = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
                            if (!enterpriseAccounts.isEmpty()) {
                                List<String> acctCodeList = enterpriseAccounts.stream().map(EnterpriseAccount::getCode).collect(Collectors.toList());
                                StringBuilder codes = new StringBuilder();
                                for (String code : acctCodeList) {
                                    codes.append(code).append(",");
                                }
                                a.setAccountCodes(StringUtils.removeEnd(codes.toString(), ","));
                            }
                        }
                    });
                }

                //todo: save config data to DB
                periodTemplateMapper.batchInsert(periodTemplateList);
                periodCellTemplateMapper.batchInsert(periodCellTemplateList);
                periodCellTemplateConfigMapper.batchInsert(periodCellTemplateConfigList);
            } else {
                result.setResult(true);
                result.setResultMsg("there is no templateGroup");
                return result;
            }
            result.setResult(true);
        } catch (Exception ex) {
            result.setResult(false);
            logger.error(ex.getMessage(), ex);
        }
        return result;
    }

    @Override
    public OperationResultDto generateData(String projectId, EnumServiceType serviceType, Boolean ifDeleteManualDataSource,
                                           Integer periodParam, Integer reportType, Optional<String> generator) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            if (serviceType.equals(EnumServiceType.VAT) && (periodParam == null || periodParam <= 0)) {
                operationResultDto.setResultMsg("PeriodRequiredForVAT");
                return operationResultDto;
            }

            if (serviceType != EnumServiceType.VAT) {
                // 暂不支持非CIT/VAT service批量后端生成报表
                throw new NotSupportedException();
            }

            int period = periodParam;
            String serviceTypeStr = serviceType.getCode().toString();

            ProjectServiceTypeExample projectServiceTypeExample = new ProjectServiceTypeExample();
            projectServiceTypeExample.createCriteria().andServiceTypeIdEqualTo(serviceTypeStr).andProjectIdEqualTo(projectId);
            Optional<Long> templateGroupId = projectServiceTypeMapper.selectByExample(projectServiceTypeExample).stream()
                    .map(ProjectServiceType::getTemplateGroupId).findFirst();
            if (templateGroupId == null) {
                operationResultDto.setResultMsg("TemplateGroupNotExist");
                return operationResultDto;
            }

            List<Long> templateIds;
            if (reportType != null) {
                PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
                periodTemplateExample.createCriteria()
                        .andTemplateGroupIdEqualTo(templateGroupId.get())
                        .andProjectIdEqualTo(projectId)
                        .andPeriodEqualTo(period)
                        .andIsActiveAssociationEqualTo(true)
                        .andReportTypeEqualTo(reportType);
                templateIds = periodTemplateMapper.selectByExample(periodTemplateExample).stream().map(PeriodTemplate::getId).collect(Collectors.toList());
            } else {
                PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
                periodTemplateExample.createCriteria().andTemplateGroupIdEqualTo(templateGroupId.get())
                        .andProjectIdEqualTo(projectId)
                        .andPeriodEqualTo(period).andIsActiveAssociationEqualTo(true);
                templateIds = periodTemplateMapper.selectByExample(periodTemplateExample).stream().map(PeriodTemplate::getId).collect(Collectors.toList());
            }

            String rslt = reportGenerator.generateData(projectId, templateIds, ifDeleteManualDataSource, null, periodParam, generator);
            if (StringUtils.isBlank(rslt)) {
                operationResultDto.setResultMsg("ReportGenerateFailed!");
                return operationResultDto;
            }
            operationResultDto.setResult(true);
        } catch (Exception ex) {
            operationResultDto.setResult(false);
            logger.error(ex.getMessage(), ex);
        }
        return operationResultDto;
    }

    @Override
    public List<CellTemplateReferenceDto> getTemplateReferences(int period) {
        return new ArrayList<>();
    }

    @Override
    public OperationResultDto<ReportDataDto> getCellData(Long reportId, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        ReportDataDto dataDto = new ReportDataDto();
        try {
            PeriodReport report = periodReportMapper.selectByPrimaryKey(reportId);
            if (report == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoReport");
                return resultDto;
            }

            PeriodTemplateExample example = new PeriodTemplateExample();
            example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(report.getTemplateId()).andPeriodEqualTo(report.getPeriod());
            Optional<PeriodTemplate> reportTemplate = periodTemplateMapper.selectByExample(example).stream().findFirst();
            if (!reportTemplate.isPresent()) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoReportTemplate");
                return resultDto;
            }

            Project project = projectMapper.selectByPrimaryKey(report.getProjectId());
            if (project == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("NoProject");
                return resultDto;
            }

            PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
            periodTemplateExample.createCriteria()
                    .andProjectIdEqualTo(projectId)
                    .andTemplateGroupIdEqualTo(reportTemplate.get().getTemplateGroupId())
                    .andPeriodEqualTo(report.getPeriod())
                    .andIsActiveAssociationEqualTo(true);

            List<Long> templateIds = new ArrayList<>();
            templateIds.add(report.getTemplateId());
            List<CellCalcInfoDto> cellCfgList = reportGenerator.getCellCalcInfo(templateIds, report.getPeriod(), project.getId());

            PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
            cellDataExample.createCriteria().andPeriodEqualTo(report.getPeriod()).andProjectIdEqualTo(projectId).andReportIdEqualTo(reportId);

            List<PeriodCellData> currentCellDataList = periodCellDataMapper.selectByExample(cellDataExample);

            PeriodFormulaBlockExample periodFormulaBlockExample = new PeriodFormulaBlockExample();
            periodFormulaBlockExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(report.getPeriod())
                    .andReportIdEqualTo(reportId);
            List<PeriodFormulaBlock> formulaBlocks = periodFormulaBlockMapper.selectByExample(periodFormulaBlockExample);
            dataDto.setFormulaBlocks(formulaBlocks);
            //todo: 关键字数据源稍后再加
            // 获取通过KeyValueDataId关联的数据源，即公式数据源

            //todo: 模型数据源稍后再加
            // 获取通过ModelBackFillAmount关联的数据源，即模型数据源
            // TODO: Check模型回填数据源是否会有多对多的情况出现，如有需distinct

            //公式计算数据源 20180711 完成
            List<DataSourceExtendDto> manualDataSourceList = periodDataSourceMapper.getFormulaDataSource(reportId);
            //loop the datasource to add datasource detail information
            for (DataSourceExtendDto dseo : manualDataSourceList) {
                //只处理有明细数据的DataSource
                if (dseo.getType().equals(FormulaDataSourceType.InputInvoice.getCode())
                        || dseo.getType().equals(FormulaDataSourceType.OutputInvoice.getCode())
                        || dseo.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
                    //todo: add json datasource detail @ here. not only itemValue
                    List<String> itemValues = new ArrayList<>();
                    itemValues.add(getDataSourceDetailList(dseo.getId()).getData());
                    dseo.setItems(itemValues);
                } else if (dseo.getType().equals(FormulaDataSourceType.Report.getCode())) {
                    List<String> itemValues = new ArrayList<>();
                    //todo: get the datasource detail to add the items property
                    PeriodDataSourceDetailExample dataSourceDetailExample = new PeriodDataSourceDetailExample();
                    dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(dseo.getId());
                    List<PeriodDataSourceDetail> dataSourceDetailList = SpringContextUtil.periodDataSourceDetailMapper.selectByExample(dataSourceDetailExample);
                    dataSourceDetailList.forEach(a -> {
                        if (StringUtils.isNotBlank(a.getItemValue())) {
                            itemValues.add(a.getItemValue());
                        }
                    });
                    if (dataSourceDetailList.size() > 0 && dataSourceDetailList.get(0).getDataSourceType() != 6) {
                        dseo.setDataSourceType(convertType(dataSourceDetailList.get(0).getDataSourceType()));
                    }
                    dseo.setItems(itemValues);
                }
            }

            List<DataSourceDtoExtend> datasource = new ArrayList<>();
            manualDataSourceList.forEach(a -> {
                DataSourceDto dataSourceDto = new DataSourceDto();
                dataSourceDto.setId(a.getId().toString());
                dataSourceDto.setAmount(a.getAmount());
                dataSourceDto.setName(a.getName());
                dataSourceDto.setOperationType(a.getOperationType());
                dataSourceDto.setDescription(a.getDescription());
                dataSourceDto.setCreateTime(a.getCreateTime());
                if (a.getDataSourceType() != null && a.getDataSourceType() > 0) {
                    dataSourceDto.setType(convetToFType(a.getDataSourceType()));
                } else {
                    dataSourceDto.setType(a.getType());
                }
                dataSourceDto.setItems(a.getItems());
                dataSourceDto.setPeriod(a.getPeriod());
                dataSourceDto.setRowIndex(a.getRowIndex());
                dataSourceDto.setRowName(a.getRowName());
                dataSourceDto.setColumnIndex(a.getColumnIndex());
                dataSourceDto.setColumnName(a.getColumnName());
                dataSourceDto.setReportTemplateId(a.getReportTemplateId().toString());

                if (a.getType().equals(FormulaDataSourceType.Report.getCode())) {
                    if (a.getItems().size() == 0) {
                        dataSourceDto.setDataSourceType(CellDataSourceType.Formula.getCode());
                    } else {
                        dataSourceDto.setDataSourceType(a.getDataSourceType());
                    }
                } else if (a.getType().equals(FormulaDataSourceType.KeyInSource.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.KeyIn.getCode());
                } else if (a.getType().equals(FormulaDataSourceType.InputInvoice.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.InputInvoice.getCode());
                } else if (a.getType().equals(FormulaDataSourceType.Voucher.getCode())) {
                    dataSourceDto.setDataSourceType(CellDataSourceType.Voucher.getCode());
                } else {
                    dataSourceDto.setDataSourceType(0);
                }

                DataSourceDtoExtend dataSourceDtoExtend = new DataSourceDtoExtend();
                dataSourceDtoExtend.setItem1(a.getCellTemplateId().toString());
                dataSourceDtoExtend.setItem2(dataSourceDto);

                datasource.add(dataSourceDtoExtend);
            });
            dataDto.setManualDataSources(datasource);

            List<CellDataDto> cellDataDtoList = new ArrayList<>();
            cellCfgList.forEach(a -> {
                CellDataDto cellDataDto = new CellDataDto();
                cellDataDto.setColumnIndex(a.getColumnIndex());
                cellDataDto.setColumnName(a.getColumnName());
                cellDataDto.setRowIndex(a.getRowIndex());
                cellDataDto.setRowName(a.getRowName());
                cellDataDto.setCellTemplateId(a.getCellTemplateId().toString());
                cellDataDto.setReportTemplateId(a.getReportTemplateId().toString());
                cellDataDto.setDataType(a.getDataType());
                cellDataDto.setReportId(reportId.toString());
                cellDataDto.setIsReadOnly(a.getIsReadOnly());
                Optional<PeriodCellData> celldata = currentCellDataList.stream()
                        .filter(x -> x.getCellTemplateId().toString().equals(a.getCellTemplateId())).findFirst();
                celldata.ifPresent(cellData -> cellDataDto.setCellId(cellData.getId().toString()));
                cellDataDto.setFormula(a.getFormula());
                celldata.ifPresent(cellData -> cellDataDto.setCellValue(cellData.getData()));
                celldata.ifPresent(cellData -> cellDataDto.setFormulaExp(cellData.getFormulaExp()));
                cellDataDto.setCellTemplateConfig(CellConfigTranslater.getConfigDto(a));
                List<DataSourceDtoExtend> entryList = datasource.stream()
                        .filter(y -> y.getItem1().equals(a.getCellTemplateId())).collect(Collectors.toList());
                List<DataSourceDto> dataSourceDtoList = new ArrayList<>();
                entryList.forEach(z -> {
                    if (z.getItem2().getItems() != null && !z.getItem2().getItems().isEmpty() && z.getItem2().getItems().get(0).contains("tag")) {
                        z.getItem2().getItems().forEach(m -> {
                            ReportCellDataSourceDto dto = JSON.parseObject(m, ReportCellDataSourceDto.class);
                            dataSourceDtoList.add(dto);
                        });
                    } else
                        dataSourceDtoList.add(z.getItem2());
                });
                cellDataDto.setDataSourceList(dataSourceDtoList);
                cellDataDtoList.add(cellDataDto);
            });

            ModifiedReportCellExample modifiedReportCellExample = new ModifiedReportCellExample();
            modifiedReportCellExample.createCriteria().andReportIdEqualTo(reportId);
            List<ModifiedReportCell> modifiedReportCellList = modifiedReportCellMapper.selectByExample(modifiedReportCellExample);
            modifiedReportCellList.forEach(mrc -> {
                Optional<CellDataDto> cdd = cellDataDtoList.stream()
                        .filter(cellDataDto -> cellDataDto.getReportId().equals(mrc.getReportId())
                                && cellDataDto.getRowIndex() == mrc.getRow()
                                && cellDataDto.getColumnIndex() == mrc.getCol()).findFirst();
                if (cdd.isPresent()) {
                    if (!mrc.getOriginalValue().equals(mrc.getValue())) {
                        cdd.get().setModifiedReportCell(mrc);
                        cdd.get().setCellValue(mrc.getValue());
                        cdd.get().setIsModified(true);
                    }
                }
            });
            dataDto.setCellData(cellDataDtoList);
            resultDto.setData(dataDto);
            resultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
        }
        return resultDto;
    }

    private Integer convertType(Integer dataSourceType) {
        if (dataSourceType.equals(FormulaDataSourceDetailType.InputInvoiceDataSourceDto.getCode())) {
            return CellDataSourceType.InputInvoice.getCode();
        } else {
            return CellDataSourceType.Formula.getCode();
        }
    }

    private Integer convetToFType(Integer cellDataSourceType) {
        if (cellDataSourceType.equals(CellDataSourceType.InputInvoice.getCode())) {
            return FormulaDataSourceType.InputInvoice.getCode();
        } else {
            return FormulaDataSourceType.Other.getCode();
        }
    }

    @Override
    public OperationResultDto<ReportDto> getReportByTemplate(Long templateId, Integer period, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        try {
            PeriodReportExample example = new PeriodReportExample();
            example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(templateId).andPeriodEqualTo(period);
            Optional<PeriodReport> report = periodReportMapper.selectByExample(example).stream().findFirst();
            if (report.isPresent()) {
                resultDto.setResult(true);
                ReportDto reportDto = new ReportDto();
                CommonUtils.copyProperties(report, reportDto);
                resultDto.setData(reportDto);
            } else {
                resultDto.setResult(false);
                resultDto.setResultMsg("can not find report");
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage());
            resultDto.setResult(false);
        }
        return resultDto;
    }

    @Override
    public OperationResultDto<PeriodCellTemplateConfig> getCellTemplateConfig(Long reportTemplateId, Integer periodParam, int rowIndex, int columnIndex) {
        OperationResultDto resultDto = new OperationResultDto();
        try {
            int period = periodParam != null ? periodParam : 0;

            if (reportTemplateId == null || reportTemplateId == 0L) {
                resultDto.setResult(false);
                resultDto.setResultMsg("report template id is invalid");
                return resultDto;
            }

            PeriodCellTemplateConfig periodCellTemplateConfig = periodCellTemplateConfigMapper.getConfigWithReportTemplateId(reportTemplateId, period, rowIndex, columnIndex);
            if (periodCellTemplateConfig == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("no config for this template!");
                return resultDto;
            } else {
                resultDto.setResult(true);
                resultDto.setData(periodCellTemplateConfig);
                return resultDto;
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
            return resultDto;
        }
    }

    @Override
    public OperationResultDto<List<VatEnterpriseAccount>> getStdAccountByIndustry(String industryId, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();
        try {
            PeriodEnterpriseAccountExample example = new PeriodEnterpriseAccountExample();
            example.createCriteria().andIsActiveEqualTo(true).andProjectIdEqualTo(projectId);
            periodEnterpriseAccountMapper.selectByExample(example);
            List<PeriodEnterpriseAccount> vatEnterpriseAccounts = periodEnterpriseAccountMapper.selectByExample(example);
            if (vatEnterpriseAccounts == null) {
                resultDto.setResult(false);
                resultDto.setResultMsg("there is no data");
                return resultDto;
            } else {
                resultDto.setResult(true);
                resultDto.setData(vatEnterpriseAccounts);
                return resultDto;
            }
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
            return resultDto;
        }
    }

    @Override
    public OperationResultDto<List<VatEnterpriseAccountResultDto>> getCellAccountRange(Long reportTemplateId, Integer periodParam, int rowIndex, int columnIndex, String projectId) {
        OperationResultDto resultDto = new OperationResultDto();

        try {
            int period = periodParam == null ? 0 : periodParam;

            PeriodCellTemplateConfig periodCellTemplateConfig =
                    periodCellTemplateConfigMapper.getConfigWithReportTemplateId(reportTemplateId, period, rowIndex, columnIndex);
            if (periodCellTemplateConfig == null || StringUtils.isBlank(periodCellTemplateConfig.getAccountCodes())) {
                resultDto.setResult(false);
                return resultDto;
            }

            PeriodEnterpriseAccountExample example = new PeriodEnterpriseAccountExample();
            example.createCriteria().andAcctCodeIn(Arrays.asList(periodCellTemplateConfig.getAccountCodes().split(","))).andProjectIdEqualTo(projectId);
            List<PeriodEnterpriseAccount> vatEnterpriseAccounts = periodEnterpriseAccountMapper.selectByExample(example);
            List<VatEnterpriseAccountResultDto> vatEnterpriseAccountResultDtos = new ArrayList<>();
            for (PeriodEnterpriseAccount vatEnterpriseAccount : vatEnterpriseAccounts) {
                VatEnterpriseAccountResultDto vatEnterpriseAccountResultDto = new VatEnterpriseAccountResultDto();
                vatEnterpriseAccountResultDto.setPeriodCellTemplateConfigId(periodCellTemplateConfig.getCellTemplateConfigId());
                vatEnterpriseAccountResultDto.setAcctCode(vatEnterpriseAccount.getAcctCode());
                vatEnterpriseAccountResultDto.setAcctPropName(vatEnterpriseAccount.getName());
                vatEnterpriseAccountResultDto.setDirectionName(vatEnterpriseAccount.getDirection() == 1 ? "借方" : "贷方");
                vatEnterpriseAccountResultDto.setAcctPropName(getAcctProp(vatEnterpriseAccount.getAcctProp()));
                vatEnterpriseAccountResultDtos.add(vatEnterpriseAccountResultDto);
            }

            resultDto.setData(vatEnterpriseAccountResultDtos);
            resultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            resultDto.setResult(false);
        }
        return resultDto;
    }

    @Override
    public OperationResultDto addCellManualDataSource(ManualDataSourceDto data, String projectId) {
        OperationResultDto operationResultDto = new OperationResultDto();

        try {
            if (data.getCellId() == null) {
                //todo: insert celldata for manual datasource
                PeriodCellData cellData = new PeriodCellData();
                cellData.setId(distributedIdService.nextId());
                cellData.setReportId(data.getReportId());
                cellData.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                cellData.setData(data.getAmount().toString());
                cellData.setFormulaExp(data.getAmount().toString());
                cellData.setCreateBy("admin");
                cellData.setCreateTime(new Date());
                cellData.setUpdateBy("admin");
                cellData.setUpdateTime(new Date());
                cellData.setProjectId(projectId);
                cellData.setPeriod(data.getPeriod());
                periodCellDataMapper.insertSelective(cellData);
                data.setCellId(cellData.getId());
            } else {
                PeriodCellData cellData = periodCellDataMapper.selectByPrimaryKey(data.getCellId());
                if (cellData.getData() != data.getAmount().toString()) {
                    cellData.setData(data.getAmount().toString());
                    periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                }
            }

            List<DataSourceExtendDto> dataSourceExtendDtos = periodDataSourceMapper.getManualDataSource(data.getCellId());

            PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
            periodCellTemplateExample.createCriteria().andCellTemplateIdEqualTo(Long.parseLong(data.getCellTemplateId())).andPeriodEqualTo(data.getPeriod());
            Optional<PeriodCellTemplate> periodCellTemplate = periodCellTemplateMapper.selectByExample(periodCellTemplateExample).stream().findFirst();

            BigDecimal originalAmount = new BigDecimal("0");
            PeriodDataSource dataSourceModel = null;
            if (dataSourceExtendDtos.size() > 0) {
                dataSourceModel = dataSourceExtendDtos.get(0).getDataSource();
                updateCellValueForDataSourceChange(dataSourceModel, data.getAmount());
                originalAmount = dataSourceModel.getAmount() != null ? dataSourceModel.getAmount() : new BigDecimal("0");
                dataSourceModel.setName(data.getName());
                dataSourceModel.setDescription(data.getDescription());
                dataSourceModel.setAmount(data.getAmount());
                dataSourceModel.setUpdateBy("Admin");
                dataSourceModel.setUpdateTime(new Date());
                periodDataSourceMapper.updateByPrimaryKeySelective(dataSourceModel);
            } else {
                dataSourceModel = new PeriodDataSource();
                Long cellDataSourceId = distributedIdService.nextId();
                dataSourceModel.setId(cellDataSourceId);
                dataSourceModel.setType(FormulaDataSourceType.KeyInSource.getCode());
                dataSourceModel.setName(data.getName());
                dataSourceModel.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                dataSourceModel.setDescription(data.getDescription());
                dataSourceModel.setAmount(data.getAmount());
                if (periodCellTemplate.isPresent()) {
                    dataSourceModel.setRowIndex(periodCellTemplate.get().getRowIndex());
                    dataSourceModel.setColumnIndex(periodCellTemplate.get().getColumnIndex());
                }
                dataSourceModel.setCreateTime(new Date());
                dataSourceModel.setUpdateTime(new Date());
                dataSourceModel.setCreateBy("admin");
                dataSourceModel.setUpdateBy("admin");
                dataSourceModel.setPeriod(data.getPeriod());

                PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                cellDataSource.setId(distributedIdService.nextId());
                cellDataSource.setCellDataId(data.getCellId());
                cellDataSource.setDataSourceId(cellDataSourceId);
                cellDataSource.setCellTemplateId(Long.parseLong(data.getCellTemplateId()));
                cellDataSource.setCreateTime(new Date());
                cellDataSource.setUpdateTime(new Date());
                cellDataSource.setProjectId(projectId);
                cellDataSource.setPeriod(data.getPeriod());
                updateCellValueForDataSourceChange(cellDataSource.getCellDataId(), new BigDecimal("0"), data.getAmount());
                periodDataSourceMapper.insertSelective(dataSourceModel);
                periodCellDataSourceMapper.insertSelective(cellDataSource);
                dataSourceExtendDtos = periodDataSourceMapper.getManualDataSource(data.getCellId());
            }

            //todo: update the reference cell's data, such as if formula is ND(1)+A2,
            //todo: if A2 is changed manually, ND(1)+A2 result to changed, and datasource of A2 should be updated
            //1. use celltemplateid and period get template id, the change should be in the same sheet.if not in the same sheet, the report should be recalculate, can't changed in time.
            //2. use the celltemplateid get col and row number, get the cell name such as A2,B3 etc.
            String cellName = StringUtils.EMPTY;
            Long templateId = 0L;
            if (periodCellTemplate.isPresent()) {
                Integer colNum = periodCellTemplate.get().getColumnIndex();
                Integer rowNum = periodCellTemplate.get().getRowIndex() + 1;
                cellName = FormulaHelper.excelColIndexToStr(colNum + 1) + rowNum.toString();
                templateId = periodCellTemplate.get().getReportTemplateId();
            }
            //3. use the template id get the reportid, use the reportid get the celltemplateids, then use the celltemplateids and cell name get the datasource which should be updated.
            PeriodReportExample reportExample = new PeriodReportExample();
            reportExample.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(templateId).andPeriodEqualTo(data.getPeriod());
            Optional<PeriodReport> report = periodReportMapper.selectByExample(reportExample).stream().findFirst();
            if (report.isPresent()) {
                Long reportId = report.get().getId();
                PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
                cellDataExample.createCriteria().andReportIdEqualTo(reportId);
                List<PeriodCellData> cellDataList = periodCellDataMapper.selectByExample(cellDataExample);
                for (PeriodCellData cellData : cellDataList) {
                    if (cellData.getId() != data.getCellId() && cellData.getFormulaExp().contains(cellName)) {
                        PeriodCellDataSourceExample cellDataSourceExample = new PeriodCellDataSourceExample();
                        cellDataSourceExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(cellData.getPeriod()).andCellDataIdEqualTo(cellData.getId()).andCellTemplateIdEqualTo(cellData.getCellTemplateId());
                        List<PeriodCellDataSource> cellDataSourceList = periodCellDataSourceMapper.selectByExample(cellDataSourceExample);
                        for (PeriodCellDataSource cellDataSource : cellDataSourceList) {
                            Long dataSourceId = cellDataSource.getDataSourceId();
                            PeriodDataSource dataSource = periodDataSourceMapper.selectByPrimaryKey(dataSourceId);
                            if (dataSource.getDescription().equals(cellName)) {
                                BigDecimal changeValue = data.getAmount().subtract(dataSource.getAmount());
                                dataSource.setAmount(data.getAmount());
                                dataSource.setUpdateBy("admin");
                                dataSource.setUpdateTime(new Date());
                                periodDataSourceMapper.updateByPrimaryKeySelective(dataSource);
                                if (!cellData.getData().equals("#VALUE!")) {
                                    cellData.setData(new BigDecimal(cellData.getData()).add(changeValue).toString());
                                } else {
                                    cellData.setData(new BigDecimal("0.0").add(changeValue).toString());
                                }
                                //cellData.setFormulaExp(cellData.getData());
                                cellData.setUpdateTime(new Date());
                                periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                            }
                        }
                    }
                }
            }
            //4. before you do the setp3 ,you should get the changed value of datasource. then update the celldata use this change. just add the change value to celldata amount.
            //5. then save the datasource and save the celldata.
            data.setId(dataSourceExtendDtos.get(0).getId());
            operationResultDto.setResult(true);
            operationResultDto.setData(data);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    @Override
    public OperationResultDto<String> addDataSource(DataSourceDto datasourceDto) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            PeriodCellData cellData = periodCellDataMapper.selectByPrimaryKey(Long.parseLong(datasourceDto.getCellDataId() != null ? datasourceDto.getCellDataId() : "0"));
            if (cellData == null) {
                operationResultDto.setResult(false);
                operationResultDto.setResultMsg("NoCellData");
                return operationResultDto;
            }

            PeriodDataSource dataSource = new PeriodDataSource();
            CommonUtils.copyProperties(datasourceDto, dataSource);
            dataSource.setType(datasourceDto.getDataSourceType());
            if (dataSource.getId() == null || dataSource.getId() == 0L) {
                dataSource.setId(distributedIdService.nextId());
                dataSource.setCreateTime(new Date());
                dataSource.setCellTemplateId(Long.parseLong(datasourceDto.getCellTemplateId()));
                dataSource.setUpdateTime(new Date());
                dataSource.setUpdateBy(datasourceDto.getUpdater());
                dataSource.setCreateBy(datasourceDto.getCreator());
                dataSource.setPeriod(datasourceDto.getPeriod());
                dataSource.setProjectId(datasourceDto.getProjectId());
            }
            dataSource.setCreateTime(new Date());
            periodDataSourceMapper.insertSelective(dataSource);

            PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
            cellDataSource.setId(distributedIdService.nextId());
            cellDataSource.setCellDataId(cellData.getId());
            cellDataSource.setCellTemplateId(cellData.getCellTemplateId());
            cellDataSource.setOperationType(datasourceDto.getOperationType());
            cellDataSource.setDataSourceId(dataSource.getId());
            cellDataSource.setCreateTime(new Date());
            cellDataSource.setUpdateTime(new Date());
            cellDataSource.setPeriod(dataSource.getPeriod());
            cellDataSource.setProjectId(dataSource.getProjectId());
            periodCellDataSourceMapper.insertSelective(cellDataSource);

            operationResultDto.setData(dataSource.getId().toString());
            operationResultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    @Override
    public OperationResultDto<String> getAllDataItems(Integer dataSourceType) {
        OperationResultDto operationResultDto = new OperationResultDto();
        try {
            CellDataSourceType cellDataSourceType = CellDataSourceType.getDSType(dataSourceType);
            String result = StringUtils.EMPTY;
            switch (cellDataSourceType) {
                case OutputInvoice:
                    result = JSON.toJSONString(outputVATInvoiceMapper.selectOutputInvoiceAndItem());
                    break;
                case InputInvoice:
                    result = JSON.toJSONString(inputVATInvoiceMapper.selectInputInvoiceWithItem());
                    break;
                case CustomInvoice:
                    List<CustomsInvoice> customsInvoices = customsInvoiceMapper.selectByExample(null);
                    result = JSON.toJSONString(customsInvoices);
                    break;
                case Formula:
                    break;
                case KeyIn:
                    break;
                case RelatedModel:
                    break;
                case SapDaily:
                    break;
                default:
                    break;
            }
            operationResultDto.setData(result);
            operationResultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    @Override
    public OperationResultDto addDataSourceItems(DataSourceDto dataSource) {
        OperationResultDto operationResultDto = new OperationResultDto();

        try {
            if (dataSource.getItems() == null || dataSource.getItems().isEmpty()) {
                operationResultDto.setResultMsg("NoData");
                operationResultDto.setResult(false);
                return operationResultDto;
            }

            if (dataSource == null || StringUtils.isBlank(dataSource.getId())) {
                operationResultDto.setResultMsg("DataSourceInvalid");
                operationResultDto.setResult(false);
                return operationResultDto;
            }

            PeriodDataSource datasourceEntity = periodDataSourceMapper.selectByPrimaryKey(Long.parseLong(dataSource.getId()));
            if (datasourceEntity == null) {
                operationResultDto.setResultMsg("DatasourceInvalid");
                operationResultDto.setResult(false);
                return operationResultDto;
            }
            updateCellValueForDataSourceChange(datasourceEntity, dataSource.getAmount());
            periodDataSourceMapper.updateByPrimaryKeySelective(datasourceEntity);
            //todo:MarkProjectModelDirty(dataSource.projectId, dataSource.serviceTypeId, dataSource.Updater);

            List<PeriodDataSourceDetail> dataSourceItems = new ArrayList<>();
            dataSource.getItems().forEach(a -> {
                PeriodDataSourceDetail dataSourceDetail = new PeriodDataSourceDetail();
                dataSourceDetail.setId(distributedIdService.nextId());
                dataSourceDetail.setDataSourceId(Long.parseLong(dataSource.getId()));
                dataSourceDetail.setDataSourceType(dataSource.getDataSourceType());
                dataSourceDetail.setItemValue(a);
                dataSourceItems.add(dataSourceDetail);
            });

            PeriodDataSourceDetailExample example = new PeriodDataSourceDetailExample();
            example.createCriteria().andDataSourceIdEqualTo(Long.parseLong(dataSource.getId()));
            List<PeriodDataSourceDetail> dataSourceDetailList = periodDataSourceDetailMapper.selectByExample(example);
            //todo: TBC tomorrow
            dataSourceDetailList.forEach(a -> {
                periodDataSourceDetailMapper.deleteByPrimaryKey(a.getId());
            });

            dataSourceItems.forEach(a -> {
                periodDataSourceDetailMapper.insertSelective(a);
            });

            operationResultDto.setResult(true);
        } catch (Exception ex) {
            logger.error(ex.getMessage(), ex);
            operationResultDto.setResult(false);
        }
        return operationResultDto;
    }

    @Override
    public OperationResultDto<String> getDataSourceDetailList(Long datasSourceId) {
        OperationResultDto<String> operationResultDto = new OperationResultDto<>();
        PeriodDataSource dataSource = periodDataSourceMapper.selectByPrimaryKey(datasSourceId);
        String itemStrValue = StringUtils.EMPTY;

        if (dataSource.getType().equals(FormulaDataSourceType.InputInvoice.getCode())) {
            PeriodDataSourceDetailExample dataSourceDetailExample = new PeriodDataSourceDetailExample();
            dataSourceDetailExample.createCriteria().andDataSourceIdEqualTo(datasSourceId).andDataSourceTypeEqualTo(CellDataSourceType.InputInvoice.getCode());
            List<PeriodDataSourceDetail> dataSourceDetails = periodDataSourceDetailMapper.selectByExample(dataSourceDetailExample);
            List<InputVATInvoiceItemExtendDto> inputVATInvoiceItemExtendDtos = new ArrayList<>();
            for (PeriodDataSourceDetail dataSourceDetail : dataSourceDetails) {
                if (StringUtils.isNotBlank(dataSourceDetail.getItemValue())) {
                    InputVATInvoiceItemExtendDto inputVATInvoiceItemExtendDto = inputVATInvoiceMapper.selectInvoiceExtendDto(dataSourceDetail.getItemValue());
                    inputVATInvoiceItemExtendDtos.add(inputVATInvoiceItemExtendDto);
                }
            }
            itemStrValue = JSON.toJSONString(inputVATInvoiceItemExtendDtos);
        } else {
            itemStrValue = StringUtils.EMPTY;
        }

        operationResultDto.setData(itemStrValue);
        operationResultDto.setResult(true);
        return operationResultDto;
    }

    private void updateCellValueForDataSourceChange(Long cellDataId, BigDecimal originalAmount, BigDecimal newAmount) {
        BigDecimal aValue = newAmount != null ? newAmount : new BigDecimal("0");
        BigDecimal bValue = originalAmount != null ? originalAmount : new BigDecimal("0");

        BigDecimal amountChange = aValue.subtract(bValue);
        PeriodCellData cellData = periodCellDataMapper.selectByPrimaryKey(cellDataId);
        BigDecimal cellVal = new BigDecimal("0");
        try {
            if (cellData != null) {
                cellVal = new BigDecimal(cellData.getData());
                Integer cellDataType = periodCellTemplateMapper
                        .getDataType(cellData.getCellTemplateId()
                                , cellData.getReportId());
                cellData.setData(FormulaHelper.roundValue(cellVal.add(amountChange)
                        , KeyValueConfigResultType.values()[cellDataType], false, null).toString());
            }
        } catch (NumberFormatException ex) {
            logger.error(ex.getMessage(), ex);
        }
    }

    private void updateCellValueForDataSourceChange(PeriodDataSource dataSourceEntity, BigDecimal amount) {
        BigDecimal originalAmount = dataSourceEntity.getAmount();
        dataSourceEntity.setAmount(amount);
        BigDecimal aValue = dataSourceEntity.getAmount() != null ? dataSourceEntity.getAmount() : new BigDecimal("0");
        BigDecimal bValue = originalAmount != null ? originalAmount : new BigDecimal("0");

        BigDecimal amountChange = aValue.subtract(bValue);
        DataSourceCellDataDto dataSourceCellDataDto =
                periodDataSourceMapper.getManualDataSource2(dataSourceEntity.getId());
        BigDecimal cellVal = new BigDecimal("0");
        if (dataSourceCellDataDto != null && dataSourceCellDataDto.getCellData().getData() != null) {
            if (dataSourceCellDataDto.getCellDataSource().getOperationType().equals(EnumOperationType.Sub)) {
                amountChange = BigDecimal.ZERO.subtract(amountChange);
            }

            try {
                cellVal = new BigDecimal(dataSourceCellDataDto.getCellData().getData());
                if (StringUtils.isNotBlank(dataSourceCellDataDto.getCellData().getData())) {
                    Integer cellDataType = periodCellTemplateMapper
                            .getDataType(dataSourceCellDataDto.getCellData().getCellTemplateId()
                                    , dataSourceCellDataDto.getCellData().getReportId());
                    dataSourceCellDataDto.getCellData().setData(
                            FormulaHelper.roundValue(cellVal.add(amountChange),
                                    KeyValueConfigResultType.values()[cellDataType],
                                    false,
                                    null).toString());
                }
            } catch (NumberFormatException ex) {
                logger.error(ex.getMessage(), ex);
            }
        }
    }

    private String getAcctProp(Integer acctProp) {
        switch (acctProp) {
            case 1:
                return "资产";
            case 2:
                return "负债";
            case 3:
                return "共同";
            case 4:
                return "权益";
            case 5:
                return "成本";
            case 6:
                return "损益";
            default:
                return "";
        }
    }
}
