package pwc.taxtech.atms.vat.service.impl.report.functions;

import com.alibaba.fastjson.JSON;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.OperandResolver;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.vat.entity.PeriodDataSource;
import pwc.taxtech.atms.vat.entity.PeriodDataSourceDetail;
import pwc.taxtech.atms.vat.entity.PeriodFormulaBlock;
import pwc.taxtech.atms.vat.service.impl.FormulaAgent;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class FunctionBase {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());
    public static final BigDecimal[] defaultTaxRates = {new BigDecimal("0.17"), new BigDecimal("0.13")
            , new BigDecimal("0.11"), new BigDecimal("0.06"), new BigDecimal("0.05"), new BigDecimal("0.03")};
    protected FormulaContext formulaContext;
    final FormulaAgent agent;

    public FunctionBase(FormulaContext formulaContext) {
        this.formulaContext = formulaContext;
        this.agent = formulaContext.getFormulaAgent();
    }

    public String getStringParam(ValueEval valueEval, OperationEvaluationContext ec) {
        try {
            ValueEval v1 = OperandResolver.getSingleValue(valueEval,
                    ec.getRowIndex(),
                    ec.getColumnIndex());
            return OperandResolver.coerceValueToString(v1);
        } catch (EvaluationException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Integer getIntParam(ValueEval valueEval, OperationEvaluationContext ec) {
        try {
            ValueEval v1 = OperandResolver.getSingleValue(valueEval,
                    ec.getRowIndex(),
                    ec.getColumnIndex());
            return OperandResolver.coerceValueToInt(v1);
        } catch (EvaluationException e) {
            e.printStackTrace();
            return null;
        }
    }

    public Long getCellTemplateId(int period, OperationEvaluationContext ec) {
        return FormulaHelper
                .getCellTemplateIdWithTemplateCodeAndColumnIndexAndRowIndex(ec.getWorkbook().getSheetName(ec.getSheetIndex()), ec.getColumnIndex(), ec.getRowIndex(), period, formulaContext);
    }

    public int getYear(int parameterYear) {
        if (parameterYear <= -1) {
            return parameterYear + formulaContext.getYear();
        } else if (parameterYear == 0) {
            return formulaContext.getYear();
        } else {
            return parameterYear;
        }
    }

    public int getPeriod(int parameterPeriod, Integer yearOffset) {

        yearOffset = 0;
        if (parameterPeriod == -99) {
            return parameterPeriod;
        } else if (parameterPeriod <= -1) {
            int period = parameterPeriod + formulaContext.getPeriod();
            while (period <= 0) {
                yearOffset--;
                period += 12;
            }
            return period;
        } else if (parameterPeriod == 0) {
            return formulaContext.getPeriod();
        } else {
            return parameterPeriod;
        }
    }

    public Long saveDataSource(OperationEvaluationContext ec, List<Object> dataSourceList,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val, int period, Long templateGroupId, String projectId) {
        return saveDataSource(ec, dataSourceList, formulaDataSourceDetailType, val, period, templateGroupId, null, null, projectId);
    }

    public Long saveDataSource(OperationEvaluationContext ec, List<Object> dataSourceList,
                               FormulaDataSourceDetailType formulaDataSourceDetailType, BigDecimal val,
                               int period, Long templateGroupId, Integer colNumP, Integer rowNumP, String projectId) {
        String reportCode = ec.getWorkbook().getSheetName(ec.getSheetIndex());
        int colNum = ec.getColumnIndex();
        int rowNum = ec.getRowIndex();
        //根据这三个值还有period获取 cell_template_Id的值
        Map<String, Object> map = new HashMap<>();
        map.put("code", reportCode);
        map.put("templateGroupId", templateGroupId);
        map.put("period", period);
        map.put("columnIndex", colNum);
        map.put("rowIndex", rowNum);
        Long periodCellTemplateId = SpringContextUtil.periodCellTemplateMapper.getCellTemplateIdWithMap(map);

        Date creatime = new Date();
        PeriodDataSource dataSource = new PeriodDataSource();
        Long dataSourceId = SpringContextUtil.distributedIdService.nextId();
        dataSource.setId(dataSourceId);
        dataSource.setType(FormulaDataSourceType.Report.getCode());
        dataSource.setName("ReportDataSource");
        dataSource.setAmount(val);
        dataSource.setDescription("描述");
        dataSource.setCreateBy("Admin");
        dataSource.setUpdateBy("Admin");
        dataSource.setCreateTime(creatime);
        dataSource.setUpdateTime(creatime);
        dataSource.setRowIndex(rowNumP == null ? ec.getRowIndex() : rowNumP);
        dataSource.setColumnIndex(colNumP == null ? ec.getColumnIndex() : colNumP);
        dataSource.setRowName("行名");
        dataSource.setColumnName("列名");
        dataSource.setCellTemplateId(periodCellTemplateId);
        dataSource.setPeriod(period);
        dataSource.setProjectId(projectId);
        SpringContextUtil.periodDataSourceMapper.insertSelective(dataSource);
        for (Object obj : dataSourceList) {
            if (obj.getClass() == java.util.ArrayList.class) {
                for (Object obj2 : (ArrayList<Object>) obj) {
                    PeriodDataSourceDetail dataSourceDetail = new PeriodDataSourceDetail();
                    dataSourceDetail.setId(SpringContextUtil.distributedIdService.nextId());
                    dataSourceDetail.setDataSourceId(dataSourceId);
                    dataSourceDetail.setDataSourceType(formulaDataSourceDetailType.getCode());
                    dataSourceDetail.setItemValue(JSON.toJSONString(obj2));
                    dataSourceDetail.setPeriod(period);
                    dataSourceDetail.setProjectId(projectId);
                    SpringContextUtil.periodDataSourceDetailMapper.insertSelective(dataSourceDetail);
                }
            } else {
                PeriodDataSourceDetail dataSourceDetail = new PeriodDataSourceDetail();
                dataSourceDetail.setId(SpringContextUtil.distributedIdService.nextId());
                dataSourceDetail.setDataSourceId(dataSourceId);
                dataSourceDetail.setDataSourceType(formulaDataSourceDetailType.getCode());
                dataSourceDetail.setItemValue(JSON.toJSONString(obj));
                dataSourceDetail.setPeriod(period);
                dataSourceDetail.setProjectId(projectId);
                SpringContextUtil.periodDataSourceDetailMapper.insertSelective(dataSourceDetail);
            }
        }
        return dataSourceId;
    }

    public void saveFormulaBlock(int period, OperationEvaluationContext ec,
                                 String formulaExpression, BigDecimal val, Long dataSourceId, String projectId) {
        Long cellTemplateId = getCellTemplateId(period, ec);
        Date creatime = new Date();
        PeriodFormulaBlock periodFormulaBlock = new PeriodFormulaBlock();
        periodFormulaBlock.setId(SpringContextUtil.distributedIdService.nextId());
        periodFormulaBlock.setProjectId(projectId);
        periodFormulaBlock.setPeriod(period);
        periodFormulaBlock.setReportId(0L);
        periodFormulaBlock.setCellTemplateId(cellTemplateId);
        periodFormulaBlock.setFormulaExpression(formulaExpression);
        periodFormulaBlock.setData(val.toString());
        periodFormulaBlock.setDataSourceId(dataSourceId);
        periodFormulaBlock.setCreateBy("Admin");
        periodFormulaBlock.setCreateTime(creatime);
        periodFormulaBlock.setUpdateBy("Admin");
        periodFormulaBlock.setUpdateTime(creatime);
        SpringContextUtil.periodFormulaBlockMapper.insertSelective(periodFormulaBlock);
    }

    public void saveFormulaBlock(int period, OperationEvaluationContext ec,
                                 String formulaExpression, String val, Long dataSourceId, String projectId) {
        Long cellTemplateId = getCellTemplateId(period, ec);
        Date creatime = new Date();
        PeriodFormulaBlock periodFormulaBlock = new PeriodFormulaBlock();
        periodFormulaBlock.setProjectId(projectId);
        periodFormulaBlock.setId(SpringContextUtil.distributedIdService.nextId());
        periodFormulaBlock.setPeriod(period);
        periodFormulaBlock.setReportId(0L);
        periodFormulaBlock.setCellTemplateId(cellTemplateId);
        periodFormulaBlock.setFormulaExpression(formulaExpression);
        periodFormulaBlock.setData(val);
        periodFormulaBlock.setDataSourceId(dataSourceId);
        periodFormulaBlock.setCreateBy("Admin");
        periodFormulaBlock.setCreateTime(creatime);
        periodFormulaBlock.setUpdateBy("Admin");
        periodFormulaBlock.setUpdateTime(creatime);
        SpringContextUtil.periodFormulaBlockMapper.insertSelective(periodFormulaBlock);
    }

}
